==> Mudlib-CD.00.28.tar.gz: lib/std/player/death_sec.c <==
/*
  player/death_sec.c

  This is a subpart of player_sec.c

  Handling of what happens when the player dies and resurrects.
  Scars all also handled here.

  This file is included into player_sec.c

  Revision history:

*/
#include "/sys/formulas.h"
#include "/sys/ss_types.h"
#include "/secure/std.h"

/*
 * Prototypes
 */
void make_scar();
void modify_on_death();
int query_average();

/*
 * Function name:   second_life
 * Description:     Handles all that should happen to a dying player.
 * Argument:        Object that caused the kill.
 * Returns:         True if the living object should get a second life
 */
public int
second_life(object killer_object)
{
    if (query_wiz_level())
	return 0;

    make_scar();
    modify_on_death();

    set_m_in(F_GHOST_MSGIN);
    set_m_out(F_GHOST_MSGOUT);

    set_headache(0);
    set_intoxicated(0);
    set_stuffed(0);
    set_soaked(0);

#ifdef LOG_KILLS
    if (killer_object)
	log_file(LOG_KILLS, ctime(time()) + " " + name +
		 "(" + query_average_stat() + ")" +
		 " killed by " +
		 killer_object->short() + "(" +
		 killer_object->query_real_name() + "), creator: " +
		 creator(killer_object) + "\n");
#endif

    stop_fight(query_enemy(-1)); /* Forget all enemies */

    tell_object(this_object(), F_DEATH_MESSAGE);

    this_object()->death_sequence();
    save_me(0); /* Save the death badge if player goes linkdead. */

    return 1;
}

/*
 * Function name:   modify_on_death
 * Description:     Modifies some values (e.g. exp, stats and hp) when a
 *                  player has died.
 */
static void 
modify_on_death()
{
    int cex, il, num;

    /* We should just reduce our combat exp 
       and affect the stats accordingly.
     */
    cex = F_DIE_REDUCE_XP(query_exp_combat());
    
    for (num = 0, il = SS_STR; il <= SS_NO_EXP_STATS; il++)
    {
	if (query_base_stat(il) >= 0)
	    num++;
    }
    
    set_exp(query_exp() - cex);
    set_exp_combat(query_exp_combat() - cex);

    cex = cex / num;

    for (il = SS_STR; il <= SS_NO_EXP_STATS; il++)
    {
	if (query_base_stat(il) >= 0)
	    set_acc_exp(il, query_acc_exp(il) - cex);
    }
    acc_exp_to_stats();
    this_object()->add_exp(0, 0);

    /*
     * We should reset our hitpoints to something above 0
     */
    set_hp(F_DIE_START_HP(query_max_hp()));
}

/*
 * Function name:   reincarnate
 * Description:     Manages the reincarnation of a player
 */
public void
reincarnate()
{
    set_player_file(LOGIN_NEW_PLAYER);
    LOGIN_NEW_PLAYER->reincarnate_me();
}

/*
 * Function name:   remove_ghost
 * Description:     This is normally not called except when a poor soul has
 *		    not been fetched by Death and reincarnated.
 * Returns:         0 if the player is not a ghost
 *                  1 otherwise.
 */
public int
remove_ghost()
{
    if (!query_ghost())
	return 0;

    tell_object(this_object(), "You have just been resurrected!\n");
    say(QCTNAME(this_object()) + " is suddenly resurrected!\n");

    set_ghost(0);
    set_m_in(F_ALIVE_MSGIN);
    set_m_out(F_ALIVE_MSGOUT);

    save_me(0);
    return 1;
}

/*
 * Function name:   make_scar
 * Description:     Add a random scar to a person.
 */
static void
make_scar()
{
    if (query_average() < 10)
	return;
    set_scar(query_scar() | (1 << random(F_MAX_SCAR)));
}


/*
 * Function name:   num_scar
 * Description:     Get the number of scars a player has
 * Returns:         int holding number of scars
 */
public int
num_scar()
{
    int i, j, n,scar;

    j = 1; n = 0; scar = query_scar();

    while (i < F_MAX_SCAR)
    {
    	if (scar & j) n++;
    	j *= 2; i++;
    }
    return n;
}

/*
 * Function name:   desc_scar
 * Description:     Get the composite string holding the scar description
 * Returns:         string holding scar description.
 */
public string
desc_scar()
{
    int i, j, scar;
    string *scar_desc, *my_scars;

    scar_desc = F_SCAR_DESCS;
    scar = query_scar();
    j = 1;
    my_scars = 0;
    while(i < F_MAX_SCAR)
    {
	if (scar & j)
	{
	    if (my_scars)
		my_scars = my_scars + ({ scar_desc[i] });
	    else
		my_scars = ({ scar_desc[i] });
	}
	j *= 2;
	i += 1;
    }
    if (!my_scars) return 0;
    if (sizeof(my_scars) == 1)
	return my_scars[0];
    if (sizeof(my_scars) == 2)
	return implode(my_scars, " and ");
    return implode(slice_array(my_scars, 0, sizeof(my_scars) - 2), ", ") +
	" and " + my_scars[sizeof(my_scars) - 1];
}

==> Mudlib-CD.00.30.tar.gz: lib/std/player/death_sec.c <==
/*
 * player/death_sec.c
 *
 * This is a subpart of player_sec.c
 *
 * Handling of what happens when the player dies and resurrects.
 * Scars all also handled here.
 *
 * This file is included into player_sec.c
 *
 * Revision history:
 *
 * 25-07-1993 /Mercade : changed xp-handling by death
 */
#include "/sys/formulas.h"
#include "/sys/ss_types.h"
#include "/secure/std.h"

/*
 * Prototypes
 */
void make_scar();
void modify_on_death();
int query_average();

/*
 * Function name:   second_life
 * Description:     Handles all that should happen to a dying player.
 * Argument:        Object that caused the kill.
 * Returns:         True if the living object should get a second life
 */
public int
second_life(object killer_object)
{
    if (query_wiz_level())
	return 0;

    make_scar();
    modify_on_death();

    set_m_in(F_GHOST_MSGIN);
    set_m_out(F_GHOST_MSGOUT);

    set_headache(0);
    set_intoxicated(0);
    set_stuffed(0);
    set_soaked(0);

#ifdef LOG_KILLS
    if (killer_object)
	log_file(LOG_KILLS, ctime(time()) + " " + name +
		 "(" + query_average_stat() + ")" +
		 " killed by " +
		 killer_object->short() + "(" +
		 killer_object->query_real_name() + "), creator: " +
		 creator(killer_object) + "\n");
#endif

    stop_fight(query_enemy(-1)); /* Forget all enemies */

    tell_object(this_object(), F_DEATH_MESSAGE);

    this_object()->death_sequence();
    save_me(0); /* Save the death badge if player goes linkdead. */

    return 1;
}

/*
 * Function name:   modify_on_death
 * Description:     Modifies some values (e.g. exp, stats and hp) when a
 *                  player has died.
 */
static void 
modify_on_death()
{
    /* reduction exp, stat exp, total exp, total reduction exp */
    int rex, sex, tex, trex, il;
    float stat_factor;

    /*
     * We should just reduce our combat exp and affect the stats accordingly.
     * Every stat is lowered by ratio of the combat exp death takes and the
     * total exp. This way a 50% higher stat will loose 50% more exp points.
     * The construct with rex is used because of the fact that integers will
     * be rounded and the total exp should match the total of the stat's exp.
     */
    tex = query_exp();

    if (!tex) /* safety! we don't want division by 0 */
    {
        /* but we still need to set the hitpoints */
        set_hp(F_DIE_START_HP(query_max_hp()));
        return;
    }

    rex = 0;
    stat_factor = ( itof(F_DIE_REDUCE_XP(query_exp_combat())) / itof(tex) );

    for (il = SS_STR; il < SS_NO_EXP_STATS; il++)
    {
	if (query_base_stat(il) >= 0) /* do we need this query? */
        {
            sex = query_acc_exp(il);
            rex = ftoi(stat_factor * itof(sex) );
            trex += rex;
	    set_acc_exp(il, (sex - rex));
        }
    }

    set_exp(tex - trex);
    set_exp_combat(query_exp_combat() - trex);

    this_object()->acc_exp_to_stats();
    this_object()->add_exp(0, 0);

    /*
     * We should reset our hitpoints to something above 0
     */
    set_hp(F_DIE_START_HP(query_max_hp()));
}

/*
 * Function name:   reincarnate
 * Description:     Manages the reincarnation of a player
 */
public void
reincarnate()
{
    set_player_file(LOGIN_NEW_PLAYER);
    LOGIN_NEW_PLAYER->reincarnate_me();
}

/*
 * Function name:   remove_ghost
 * Description:     This is normally not called except when a poor soul has
 *		    not been fetched by Death and reincarnated.
 * Arguments:       quiet : true if no messages should be printed
 * Returns:         0 if the player is not a ghost
 *                  1 otherwise.
 */
public int
remove_ghost(int quiet)
{
    if (!query_ghost())
	return 0;

    if (!quiet)
    {
	tell_object(this_object(), "You have just been resurrected!\n");
	say(QCTNAME(this_object()) + " is suddenly resurrected!\n");
    }
    set_ghost(0);
    set_m_in(F_ALIVE_MSGIN);
    set_m_out(F_ALIVE_MSGOUT);

    save_me(0);
    return 1;
}

/*
 * Function name:   make_scar
 * Description:     Add a random scar to a person.
 */
static void
make_scar()
{
    if (query_average() < 10)
	return;
    set_scar(query_scar() | (1 << random(F_MAX_SCAR)));
}


/*
 * Function name:   num_scar
 * Description:     Get the number of scars a player has
 * Returns:         int holding number of scars
 */
public int
num_scar()
{
    int i, j, n,scar;

    j = 1; n = 0; scar = query_scar();

    while (i < F_MAX_SCAR)
    {
    	if (scar & j) n++;
    	j *= 2; i++;
    }
    return n;
}

/*
 * Function name:   desc_scar
 * Description:     Get the composite string holding the scar description
 * Returns:         string holding scar description.
 */
public string
desc_scar()
{
    int i, j, scar;
    string *scar_desc, *my_scars;

    scar_desc = F_SCAR_DESCS;
    scar = query_scar();
    j = 1;
    my_scars = 0;
    while(i < F_MAX_SCAR)
    {
	if (scar & j)
	{
	    if (my_scars)
		my_scars = my_scars + ({ scar_desc[i] });
	    else
		my_scars = ({ scar_desc[i] });
	}
	j *= 2;
	i += 1;
    }
    if (!my_scars) return 0;
    if (sizeof(my_scars) == 1)
	return my_scars[0];
    if (sizeof(my_scars) == 2)
	return implode(my_scars, " and ");
    return implode(slice_array(my_scars, 0, sizeof(my_scars) - 2), ", ") +
	" and " + my_scars[sizeof(my_scars) - 1];
}
